package net.pricefx.common.api;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

public enum WorkflowFormulaNature implements IFormulaNature{
    DEFAULT(null, "DEFAULT"),

    WORKFLOW_POST_STEP_LOGIC("wfStepLogic", "WFPSL");

    private static final Map<String,WorkflowFormulaNature> lookupViaDBValue = new HashMap<>(WorkflowFormulaNature.values().length + 1, 1);
    private static final Map<String,WorkflowFormulaNature> lookupViaRestApiValue = new HashMap<>(WorkflowFormulaNature.values().length + 1, 1);

    private String dbValue;
    private String restApiValue;

    static {
        for(WorkflowFormulaNature nature : EnumSet.allOf(WorkflowFormulaNature.class)) {
            lookupViaDBValue.put(nature.dbValue, nature);
        }

        for(WorkflowFormulaNature nature : EnumSet.allOf(WorkflowFormulaNature.class)){
            lookupViaRestApiValue.put(nature.restApiValue, nature);
        }
    }

    public static WorkflowFormulaNature getViaDBValue(String dbValue) {
        return lookupViaDBValue.get(dbValue);
    }

    public static WorkflowFormulaNature getViaRestApiValue(String restApiValue) {
        return lookupViaRestApiValue.get(restApiValue);
    }

    private WorkflowFormulaNature(String dbValue, String restApiValue) {
        this.dbValue = dbValue;
        this.restApiValue = restApiValue;
    }

    public String getDbValue() {
        return dbValue;
    }

    public String getRestApiValue() {
        return restApiValue;
    }

}
