package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.ComparisonType;
import net.pricefx.common.api.chart.LimitPosition;
import net.pricefx.common.api.chart.LimitType;
import net.pricefx.common.api.chart.SeriesType;
import net.pricefx.common.api.chart.SortType;
import net.pricefx.common.api.chart.WhiskerType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;


public class BarLineChart extends AbstractBuilder {

	public BarLineChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private BarLineOptions options;
	private Dictionary<BarLineChart> dictionary;
	
	public BarLineSeries addSeries() {	
		String id = addInvocation("addSeries");
		return new BarLineSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public BarLineOptions getOptions() {

		String id = addInvocation("getOptions");
		if (options == null) 
			options = new BarLineOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<BarLineChart> addAuxLine() {

		String id = addInvocation("addAuxLine");
		return new AuxLine<BarLineChart>(this, id, "AuxLine");
	}
	
	public Dictionary<BarLineChart> getDictionary() {

		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {

		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.BAR;
		return r;
	}
	
	public static class BarLineSeries extends AbstractSeries<BarLineChart,BarLineSeries> {

		public BarLineSeries(BarLineChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public BarLineSeries setHideDataLabels(Boolean hideDataLabels) {
			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
				
		public LimitToMeasure<BarLineSeries> setLimitTo(LimitPosition limitPosition, BigDecimal amount, LimitType limitType) {

			String id = addInvocation("setLimitTo", limitPosition, amount, limitType);
			return new LimitToMeasure<BarLineSeries>(this, id, "LimitToMeasure");
		}
		
		public BarLineSeries setAxisX(String axisX) {

			addInvocation("setAxisX", axisX);
			return this;
		}
		
		public AggrFieldAxis<BarLineSeries> setAxisY(String axisY) {

			String id = addInvocation("setAxisY", axisY);
			return new AggrFieldAxis<BarLineSeries>(this, id, "AxisY");
		}

		public BarLineSeries setType(SeriesType seriesType) {

			addInvocation("setType", seriesType);
			return this;
		}
		
		public BarLineSeries withSortByAxisX(SortType sortType) {

			addInvocation("withSortByAxisX", sortType);
			return this;
		}

		public BarLineSeries withSortByAxisY(SortType sortType) {

			addInvocation("withSortByAxisY", sortType);
			return this;
		}

		public BarLineSeries setComparisonDim(String comparisonDimension) {

			addInvocation("setComparisonDim", comparisonDimension);
			return this;
		}
		
		public BarLineSeries setShowBoxplot(Boolean showBoxplot) {

			addInvocation("setShowBoxplot", showBoxplot);			
			return this;
		}
		
		public NonAggrFieldAxis<BarLineSeries> setBoxplotAxis() {

			String id = addInvocation("setBoxplotAxis");
			return new NonAggrFieldAxis<BarLineSeries>(this, id, "BoxplotAxis");
		}
		
		public BarLineSeries setWhisker(WhiskerType whiskerType) {

			addInvocation("setWhisker", whiskerType);						
			return this;
		}		

		public BarLineAdditionalMeasure<BarLineSeries> addAdditionalMeasure() {

			String id = addInvocation("addAdditionalMeasure");
			return new BarLineAdditionalMeasure<BarLineSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<BarLineSeries> setFormula() {

			String id = addInvocation("setFormula");
			return new Formula<BarLineSeries>(this, id, "Formula");
		}
		
		public BarLineSeries setGeneratedQueryDto(String generatedQueryDto) {

			addInvocation("setGeneratedQueryDto", generatedQueryDto);
			return this;
		}


	}
	
	public static class BarLineOptions extends AbstractOptions<BarLineChart,BarLineOptions> {

		public BarLineOptions(BarLineChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
		
		
		public BarLineOptions setComparisonType(ComparisonType comparisonType) {

			addInvocation("setComparisonType", comparisonType);
			return this;
		}
				
		public BarLineOptions setBoxplotAxisLabel(String boxplotAxisLabel) {

			addInvocation("setBoxplotAxisLabel", boxplotAxisLabel);
			return this;
		}
		
		
		public BarLineOptions setBoxplotAxisMin(BigDecimal boxplotAxisMin) {

			addInvocation("setBoxplotAxisMin", boxplotAxisMin);
			return this;
		}
		
		public BarLineOptions setBoxplotAxisMax(BigDecimal boxplotAxisMax) {

			addInvocation("setBoxplotAxisMax", boxplotAxisMax);
			return this;
		}

		
	}
		
	public static class BarLineAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,BarLineAdditionalMeasure<ATTACHEDTO>> {

		public BarLineAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}

		
		public Expression<BarLineAdditionalMeasure<ATTACHEDTO>> withExpression() { 

			String id = addInvocation("withExpression");
			return new Expression<BarLineAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}
		
		public BarLineAdditionalMeasure<ATTACHEDTO> withBoxplotField() { 

			addInvocation("withBoxplotField");
			return this;
		}
		
		public Expression<BarLineAdditionalMeasure<ATTACHEDTO>> withBoxplotExpression() { 
			
			String id = addInvocation("withBoxplotExpression");
			return new Expression<BarLineAdditionalMeasure<ATTACHEDTO>>(this, id, "BoxplotExpression");
		}



	}
	
}