package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class DataTableChart extends AbstractBuilder {

	public DataTableChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}


	private DataTableOptions options;
	private Dictionary<DataTableChart> dictionary;
	
	public DataTableSeries addSeries() {
		String id = addInvocation("addSeries");
		return new DataTableSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public DataTableOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new DataTableOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public Dictionary<DataTableChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.DATATABLE;
		return r;
	}
	
	public static class DataTableSeries extends AbstractSeries<DataTableChart,DataTableSeries> {

		public DataTableSeries(DataTableChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		public DataTableSeries setHideDataLabels(Boolean hideDataLabels) {
			
			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
		
		public DataTableSeries addGroupBy(String value) {

			addInvocation("addGroupBy",value);
			return this;
		}
		
		public DataTableAdditionalMeasure<DataTableSeries> addAdditionalMeasure() {

			String id = addInvocation("addAdditionalMeasure");
			return new DataTableAdditionalMeasure<DataTableSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<DataTableSeries> setFormula() {

			String id = addInvocation("setFormula");
			return new Formula<DataTableSeries>(this, id, "Formula");
		}
		
		public DataTableSeries setGeneratedQueryDto(String generatedQueryDto) {

			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}
		

	}
	
	public static class DataTableOptions extends AbstractOptions<DataTableChart,DataTableOptions> {

		public DataTableOptions(DataTableChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
		
	}
		
	public static class DataTableAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,DataTableAdditionalMeasure<ATTACHEDTO>> {

		public DataTableAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}
		
		public Expression<DataTableAdditionalMeasure<ATTACHEDTO>> withExpression() { 
			
			String id = addInvocation("withExpression");
			return new Expression<DataTableAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}


	}
	
}